from mayan.apps.appearance.classes import Icon

# Document version

icon_document_version_ocr_content_single_delete = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='times'
)
icon_document_version_ocr_content_multiple_delete = icon_document_version_ocr_content_single_delete
icon_document_version_ocr_content_detail = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_version_ocr_single_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-right'
)
icon_document_version_ocr_multiple_submit = icon_document_version_ocr_single_submit

# Document version page

icon_document_version_page_ocr_content_detail = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_version_page_ocr_content_edit = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_version_ocr_content_download = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-down'
)

# Document type

icon_document_type_ocr_settings = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_type_ocr_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-right'
)
