{% load static %}

<link href="{% static 'views/css/views.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'views/node_modules/dropzone/dist/dropzone.css' %}" media="screen" rel="stylesheet" type="text/css" />

<style>
    .views-text-wrap {
        white-space: pre-wrap;
    }
</style>
