{% extends 'appearance/base.html' %}

{% load i18n %}
{% load static %}

{% load navigation_tags %}

{% block title %}{{ title }}{% endblock %}

{% block content %}

<div class="well center-block">
    <div class="appearance-flex-grid-container appearance-btn-large-container">
        {% with 'navigation/large_button_link.html' as link_template %}
            {% for menu_results in resolved_links %}
                {% with menu_results.links as object_navigation_links %}
                    {% include 'navigation/generic_navigation.html' %}
                {% endwith %}
            {% empty %}
                <p class="text-center">
                    {% include 'appearance/partials/no_results.html' %}
                </p>
            {% endfor %}
        {% endwith %}
    </div>
</div>
{% endblock %}
