from mayan.apps.appearance.classes import Icon

# Document

icon_document_link_send_single = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)
icon_document_link_send_multiple = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)

# Document file

icon_document_file_attachment_send_single = Icon(
    driver_name='fontawesome', symbol='paperclip'
)
icon_document_file_attachment_send_multiple = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)
icon_document_file_link_send_single = Icon(
    driver_name='fontawesome', symbol='paperclip'
)
icon_document_file_link_send_multiple = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)

# Document version

icon_document_version_attachment_send_multiple = Icon(
    driver_name='fontawesome', symbol='paperclip'
)
icon_document_version_attachment_send_single = Icon(
    driver_name='fontawesome', symbol='paperclip'
)
icon_document_version_link_send_multiple = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)
icon_document_version_link_send_single = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)

# Mailing profile

icon_user_mailer_backend_select = Icon(
    driver_name='fontawesome-dual', primary_symbol='envelope',
    secondary_symbol='plus'
)
icon_user_mailer_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='envelope',
    secondary_symbol='plus'
)
icon_user_mailer_delete = Icon(driver_name='fontawesome', symbol='times')
icon_user_mailer_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_user_mailer_list = Icon(driver_name='fontawesome', symbol='envelope')
icon_user_mailer_test = Icon(driver_name='fontawesome', symbol='play')
