class DeprecationWarning(UserWarning):
    """
    Warning when a feature or interface has been deprecated.
    """


class InterfaceWarning(UserWarning):
    """
    Warning when using obsolete internal interfaces.
    """
