from mayan.apps.appearance.classes import Icon

# App

icon_file_metadata = Icon(
    driver_name='fontawesome', symbol='chess-board'
)

# Document type

icon_document_type_file_metadata_settings = Icon(
    driver_name='fontawesome', symbol='chess-board'
)
icon_document_type_file_metadata_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='chess-board',
    secondary_symbol='arrow-right'
)

# Document file

icon_document_file_metadata_single_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='chess-board',
    secondary_symbol='arrow-right'
)
icon_document_file_metadata_multiple_submit = icon_document_file_metadata_single_submit
icon_file_metadata_driver_attribute_list = Icon(
    driver_name='fontawesome', symbol='chess-board'
)
icon_file_metadata_driver_list = Icon(
    driver_name='fontawesome', symbol='chess-board'
)
