# This constant may need tweaking as document upload code path changes.
# The value is targeted at making the document upload process fail exactly
# during the MIME type detection phase. Different architectures may need
# different values.
MAXIMUM_HEAP_MEMORY = 140000000
